# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010 Nexedi SA and Contributors. All Rights Reserved.
# Copyright (c) 2006-2008 Zope Corporation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

def _get_extended_section_by_source(buildout_section_dict):
  """
  From the 'buildout' section, get all the sections, which are going
  to be extended using this extension, along with their target and
  source options.

  @param buildout_section_dict: buildout section dict
  @rtype: list
  @returns: target section, target option and source option for all sections
  """
  extended_section_list = []

  for option_name, option_value in buildout_section_dict.iteritems():
    try:
      prefix, suffix = option_name.split('-section-extender-target-section')
    except ValueError:
      continue

    target_option_key = prefix + '-section-extender-target-option'
    source_option_key = prefix + '-section-extender-source-option'

    try:
      extended_section_list.append((option_value,
                                    buildout_section_dict[target_option_key],
                                    buildout_section_dict[source_option_key]))
    except KeyError:
      print "WARNING: Ignoring section '%s' with missing target section or option" % name
      continue

  return extended_section_list

def ext(buildout):
  """
  This extension allows any sections in 'parts' to define a (source)
  option whose value will be added to the (target) option of a
  (target) section, thus allowing to extend a section from another
  one.

  For each section to be extended ('PREFIX-section-extender-target-section'),
  there has to be a target option ('PREFIX-section-extender-target-option'),
  namely the name of the option in the extended section, _and_ a
  source option ('PREFIX-section-extender-source-option'), which is
  the option name set in sections.

  For example, the following buildout configuration snippet allows to
  extend the 'supervisor-instance' for the option 'programs' and can
  be found in any sections as 'supervisor-program':

  [buildout]
  foo-section-extender-target-section = supervisor-instance
  foo-section-extender-target-option = programs
  foo-section-extender-source-option = supervisor-program

  @param buildout: buildout configuration sections and their options
  @type buildout: dict
  """
  # Get the sections to be extended
  extended_section_list = _get_extended_section_by_source(buildout['buildout'])

  for part in buildout['buildout']['parts'].splitlines():
    if not part:
      continue

    for target_section, target_option, source_option in extended_section_list:
      if source_option in buildout[part]:
        # Set the value if it has never been set before
        if target_option not in buildout[target_section]:
          buildout[target_section][target_option] = buildout[part][source_option]
        # Otherwise, just add on a new line
        else:
          buildout[target_section][target_option] += '\n' + buildout[part][source_option]
